#include <avr/io.h>
#include <avr/interrupt.h>
#include "extrino_portx.h"
#include "hd44780.h"

int main(void) {
	
	PortxInit();								// inicjalizacja portu X
	LcdInit();									// inicjalizacja wywietlacza LCD
	
	// konfiguracja komparatora 0 w porcie A
	ACA.AC0MUXCTRL	=	AC_MUXPOS_PIN6_gc |		// wejcie + PIN A6
						AC_MUXNEG_SCALER_gc;	// wejcie - dzielnik napicia
	ACA.CTRLB		=	38;						// 2/3 napicia Vcc jako odniesienie
	ACA.AC0CTRL		=	AC_HYSMODE_LARGE_gc|	// dua histereza
						AC_ENABLE_bm;			// wczenie komparatora
						
	// konfiguracja komparatora 1 w porcie A
	ACA.AC1MUXCTRL	=	AC_MUXPOS_PIN6_gc |		// wejcie + PIN A6
						AC_MUXNEG_BANDGAP_gc;	// 1V bandgap rdo nap odniesienia
	ACA.AC1CTRL		=	AC_HYSMODE_LARGE_gc|	// dua histereza
						AC_ENABLE_bm;			// wczenie komparatora
						
	// konfiguracja komparatora okienkowego
	ACA.WINCTRL		=	AC_WEN_bm|				// uruchomienie trybu okienkowego
						AC_WINTMODE_INSIDE_gc|	// wybr przerwania
						AC_WINTLVL_LO_gc;		// wybr priorytety przerwania
	
	// pin do adowania/rozadowania kondensatora
	PORTA.DIRSET	=	PIN5_bm;				// A5 jako wyjcie
	PORTA.OUTSET	=	PIN5_bm;				// stan niski w celu rozadowania kond
	
	// system zdarze
	EVSYS.CH0MUX	=	EVSYS_CHMUX_ACA_CH1_gc;	// zdarzenie komparatora jest przesyane kanaem 0 systemu zdarze
	
	// konfiguracja timera
	TCC0.CTRLD		=	TC_EVACT_FRQ_gc |		// tryb pomiaru czstotliwoci
						TC_EVSEL_CH0_gc;		// zdarzenie wywouje kana 0
	TCC0.CTRLB		=	TC0_CCAEN_bm|			// przechwycenie do rejestru CCA
						TC_WGMODE_NORMAL_gc;	// zwyky tryb zliczania impulsw
	TCC0.CTRLA		=	TC_CLKSEL_DIV8_gc;		// preskaler i uruchomienie timera
	TCC0.INTCTRLA	=	TC_OVFINTLVL_LO_gc;		// pritrytet przerwania od przepenienia
	
	// konfiguracja przerwa
	PMIC.CTRL		=	PMIC_LOLVLEN_bm;		// wczenie przerwa o priorytecie LO
	sei();
	
	uint16_t wynik;								// zmienna przechowujca pojemno kondensatora
	
	while(1) {
		
		// wywietlanie wyniku
		Lcd1;
		wynik = TCC0.CCA / 31;					// komentarz w tekcie (1)
		LcdDecComma(wynik,1);					// wywietlenie liczby staoprzecinkowej
		Lcd("uF       ");
		
		// wywietlenie stanu pinu A5 na diodach PORTX
		PORTX.OUT	=	PORTA.OUT & PIN5_bm;		
	}
}

ISR(ACA_ACW_vect) {								// przerwanie komparatora
	
	if(PORTA.OUT & PIN5_bm) {					// jeeli teraz jest adowanie
		ACA.WINCTRL	=	AC_WEN_bm|				
						AC_WINTMODE_BELOW_gc|
						AC_WINTLVL_LO_gc;		
	} else {									// jeeli teraz jest rozadowywanie
		ACA.WINCTRL	=	AC_WEN_bm|				
						AC_WINTMODE_ABOVE_gc|
						AC_WINTLVL_LO_gc;
	}
	
	PORTA.OUTTGL	=	PIN5_bm	;				// zmiana stanu pinu A5 na przeciwny
}

ISR(TCC0_OVF_vect) {							// przepenienie timera C0
	PORTA.OUTTGL	=	PIN5_bm	;				// zmiana stanu pinu A5 na przeciwny
}